#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QtGlobal>
#include <stdlib.h>
#include <QDebug>
#include <QTimer>
#include <QFile>
#include <qfiledialog.h>
#include <QIODevice>
#include <QDataStream>
#include <QTextStream>
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    // 初始化串口
    m_serial = new Serial;
    timer = new QTimer();
    timerout = new QTimer();
    //IAP定时器
    timerIAP = new QTimer();
    IAP = new iap;
    // 寻找可用串口
    QStringList serialStrList;
    serialStrList = m_serial->scanSerial();
    for (int i=0; i<serialStrList.size(); i++)
    {
        ui->portComboBox->addItem(serialStrList[i]); // 在comboBox那添加串口号
    }
    // 默认设置波特率为115200（第5项）
    ui->baudComboBox->setCurrentIndex(1);

    //隐藏底部的三个组件
    //ui->label_recvstate->setVisible(false);
    ui->recvTextEdit->setVisible(false);
    ui->clearTextButton->setVisible(false);
    // 当下位机中有数据发送过来时就会响应这个槽函数
    connect(m_serial, SIGNAL(readSignal()), this, SLOT(readSerialData()));
    connect(timer, SIGNAL(timeout()), this, SLOT(timeUpdate()));
    connect(timerout, SIGNAL(timeout()), this, SLOT(timeoutUpdate()));
    connect(timerIAP, SIGNAL(timeout()), this, SLOT(timeIAPUpdate()));

    ui->iapDownLoadButton->setEnabled(false);
}

MainWindow::~MainWindow()
{
    delete ui;
}
//接收超时，使用此处清零各个标志。尤其是恢复出厂设置发送的报文flag
void MainWindow::timeoutUpdate()
{
    timerout->stop();//等待接收数据超时，则停止超时计数器
    //1.清除flag
    getMsgFlag = 0x00;
    //2.显示接收状态为超时
    ui->label_recvstate->clear();
    ui->label_recvstate->setText(tr("接收信息超时！"));
}
void MainWindow::on_openPortButton_clicked()
{
    if(ui->openPortButton->text() == tr("打开"))
        {
            if(m_serial->open(ui->portComboBox->currentText(), ui->baudComboBox->currentText().toInt()))
            {
                // 关闭下拉列表使能
                ui->portComboBox->setEnabled(false);
                ui->baudComboBox->setEnabled(false);
                // 修改按钮名称
                ui->openPortButton->setText(tr("关闭"));

                //串口打开时，设置扫描按钮禁能
                ui->scanPortButton->setEnabled(false);
            }
        }
        else
        {
            // 关闭串口
            m_serial->close();
            // 重新开启下拉列表使能
            ui->portComboBox->setEnabled(true);
            ui->baudComboBox->setEnabled(true);
            // 恢复按钮名称
            ui->openPortButton->setText(tr("打开"));
            //串口关闭时，设置扫描按钮有效
            ui->scanPortButton->setEnabled(true);
        }
}

void MainWindow::on_clearTextButton_clicked()
{
    ui->recvTextEdit->clear();
}


// 读取从自定义串口类获得的数据
void MainWindow::readSerialData()
{
    timer->start(30);
    timerout->stop();//接收到数据，则停止超时计数器
    recvPackage.append(m_serial->getReadBuf());
    m_serial->clearReadBuf(); // 读取完后，清空数据缓冲区
}
void MainWindow::timeUpdate()
{
    static int recvCnt=0;
    timer->stop();//此处的计时器用来等待100ms
    recvCnt++;
    if(recvPackage.length() != 0){
        qDebug("time out:%d",recvCnt);
        //IAP升级界面
        //检验头
        if (recvPackage.length() >= 8 && recvPackage[0] == 0xF1)
        {
            //检验crc
            QByteArray bt =IAP->calcCRC32(recvPackage, recvPackage.length() - 4);
            if (recvPackage[recvPackage.length() - 4] == bt[0] &&
                recvPackage[recvPackage.length() - 3] == bt[1] &&
                recvPackage[recvPackage.length() - 2] == bt[2] &&
                recvPackage[recvPackage.length() - 1] == bt[3])
            {
                IAP->ResponseData.resize(recvPackage.length());
                for (int i = 0; i < recvPackage.length(); i++)
                {
                    IAP->ResponseData[i] = recvPackage[i];
                }
                //显示到界面
                QString string;
                string.prepend(IAP->ByteArrayToHexString(IAP->ResponseData));
                ui->tb_IAPMsg->append(string);
            }
        }
        recvPackage.clear();
    }
}
void MainWindow::procPackage()
{
        int  _recvdata[50];
        for(int i=0;i<recvPackage.length();i++){

            _recvdata[i] = recvPackage[i]&0xff;
        }
}

void MainWindow::on_scanPortButton_clicked()
{
    // 寻找可用串口
    QStringList serialStrList;
    serialStrList = m_serial->scanSerial();

    ui->portComboBox->clear();
    qDebug("newportnum:%d",serialStrList.size());

    for (int i=0; i<serialStrList.size(); i++)//接着添加最新扫描到的
    {
        ui->portComboBox->addItem(serialStrList[i]); // 在comboBox那添加串口号
    }
}


void MainWindow::on_actionabout_triggered()
{
    QMessageBox::information(NULL, tr("关于"), tr("西安爱极客网络科技有限公司"));
}

//加载固件
void MainWindow::LoadFile()
{
    //获取固件内容
    QFile file;
    file.setFileName(IAP->fileName);
    file.open(QIODevice::ReadOnly);
    qDebug("size:%d",file.size());
    //读固件到数组
    IAP->databuf.resize(file.size());
    IAP->databuf = file.readAll();
    IAP->packet_zheng = IAP->databuf.length()/IAP->PackageSize;
    IAP->packet_yu = IAP->databuf.length()%IAP->PackageSize;
    IAP->PackageNum = IAP->packet_zheng + (IAP->packet_yu == 0 ? 0:1);
    QString str=QString("固件长度：%1\r\n包大小：%2\r\n包总数：%3").arg(file.size()).arg(IAP->PackageSize).arg(IAP->PackageNum);
    ui->tb_IAPMsg->setText(str);
}
//IAP加载固件按钮
void MainWindow::on_iapLoadFileButton_clicked()
{
    //加载固件
    qDebug("load firmware");
    //加载文件
    QString curPath=QDir::currentPath();
    //调用打开文件对话框打开一个文件
    IAP->fileName = QFileDialog::getOpenFileName(this,tr("加载一个固件"),".","固件(*.bin)");
    //如果目录为空，退出
    if (IAP->fileName.isEmpty())
     return;
    ui->b_FirmwarePath->setText(IAP->fileName);
    ui->iapDownLoadButton->setEnabled(true);
    //加载固件
    LoadFile();
}
//下载
void MainWindow::on_iapDownLoadButton_clicked()
{
    //修改下载按钮名称，切换功能
    if(ui->iapDownLoadButton->text()=="下载")
    {
        //清空消息区
        ui->tb_IAPMsg->clear();
        //加载固件
        LoadFile();
        ui->iapDownLoadButton->setText(tr("取消"));
        //标记刷固状态
        IAP->iapState = 1;
        IAP->ResponseData.clear();
        //开启定时器，执行刷固流程
        timerIAP->setInterval(50);
        timerIAP->start();
    }
    else
    {
        ui->iapDownLoadButton->setText(tr("下载"));
        //停止刷固件
        timerIAP->stop();
        //标记刷固状态
        IAP->iapState = 0;
        ui->lcdNumber->display(0);
    }
}
//清空接收区
void MainWindow::on_iapClearButton_clicked()
{
    ui->tb_IAPMsg->clear();
}
//iap定时器槽函数，单位时间进入一次
void MainWindow::timeIAPUpdate()
{
    //停止计时
    timerIAP->stop();
    //正在请求刷固
    if(IAP->iapState == 1)
    {
        //判断是否响应了刷固件请求
        if (IAP->ResponseData[1] == 0x81)
        {
            //清除接收缓存
            IAP->ResponseData.clear();
            //进入刷固件流程
            IAP->iapState = 2;
            IAP->nowIndex = 0;
            IAP->timeCount = 0;//清空超时计数
            ui->tb_IAPMsg->append(tr("收到响应，进入刷固件流程！"));
            QByteArray data = IAP->send_data_zheng(IAP->nowIndex);
            m_serial->sendData(data);
            ui->tb_IAPMsg->append(tr("已发送第%1包，共%2包").arg(IAP->nowIndex+1).arg(IAP->PackageNum));

        }
        else
        {
            //发送请求刷固帧
            QByteArray data = IAP->send_Ask();
            m_serial->sendData(data);
            ui->tb_IAPMsg->append(tr("请求更新固件！"));
        }
    }
    else
    //正在刷固件
    if(IAP->iapState == 2)
    {
        IAP->timeCount++;
        //收到响应帧
        if (IAP->ResponseData[1] == 0x82)
        {
            int idx = (IAP->ResponseData[2] << 8) | IAP->ResponseData[3];
            //包索引是否一致
            if (IAP->nowIndex == idx)
            {
                //判断结果
                switch (IAP->ResponseData[4])
                {
                    case 0x01:
                        //正常
                        if ((idx + 1) == IAP->PackageNum)
                        {
                            //升级完成
                            ui->tb_IAPMsg->append(tr("Flash写入成功！"));
                            ui->tb_IAPMsg->append(tr("更新完成！"));
                            ui->lcdNumber->display(0);
                            IAP->iapState = 3;
                        }
                        else
                        //整包已经发完
                        if ((idx + 1) == IAP->packet_zheng)
                        {
                            //开始发最后一包
                            IAP->nowIndex ++;
                            QByteArray data = IAP->send_data_yu();
                            m_serial->sendData(data);
                            ui->tb_IAPMsg->append(tr("已发送第%1包，共%2包").arg(IAP->nowIndex+1).arg(IAP->PackageNum));
                        }
                        else
                        {
                            //继续下一包发送
                            IAP->nowIndex ++;
                            QByteArray data = IAP->send_data_zheng(IAP->nowIndex);
                            m_serial->sendData(data);
                            ui->tb_IAPMsg->append(tr("已发送第%1包，共%2包").arg(IAP->nowIndex+1).arg(IAP->PackageNum));
                        }
                        break;
                    case 0x02:
                    case 0x03:
                        //数据错误，重新发送
                        ui->tb_IAPMsg->append(tr("数据错误，重新发送!"));
                        QByteArray data = IAP->send_data_zheng(IAP->nowIndex);
                        m_serial->sendData(data);
                        IAP->timeCount = 0;
                        break;
                }
            }
        }
        else
        //没有收到响应帧
        {
            //判断是否超时
            if (IAP->timeCount > IAP->timeout)
            {
                //等待超时，重发
                ui->tb_IAPMsg->append(tr("等待超时，重发!"));
                QByteArray data ;
                //如果是余包
                if((IAP->nowIndex+1)==IAP->PackageNum && IAP->packet_zheng!=IAP->PackageNum)
                {
                   data = IAP->send_data_yu();
                }
                else
                   data = IAP->send_data_zheng(IAP->nowIndex);
                m_serial->sendData(data);
                IAP->timeCount = 0;
            }
        }
    }

    //通信计数
    int val= ui->lcdNumber->value();
    ui->lcdNumber->display(val+1);
    if(IAP->iapState == 1||IAP->iapState == 2)
    {
        //开始计时
        timerIAP->start();
    }
    else
    if(IAP->iapState == 0 ||IAP->iapState == 3)
    {
       ui->iapDownLoadButton->setText(tr("下载"));
    }
}
